; QL Floppy Disk read sector      1990     Tony Tebby  QJUMP      Jo

        section fq

        xdef    fq_rsecp                 ; read sector from Floppy disk
        xdef    fq_rsecl                 ; read sector from Floppy disk
        xdef    fq_rsec                  ; read sector

        xref    fq_setp
        xref    fq_sss
        xref    fq_rest
        xref    fq_cmd
        xref    fq_stat
        xref    fq_ckrw
        xref    fq_read1s               ;+

        include 'keys_karte'       ;+
        include 'keys_iod'         ;+
        include 'keys_err'
        include 'dd_fq_ddlk'
        include 'mac_assert'
;+++
; This routine reads a sector from a QL Floppy disk
;
;       d0 cr   logical block (group/sector) / error code
;       d6 c  p msw drive number (1 to 8)
;       a1 c  p dma address
;       a2  r   ioadr WD1772    ;+
;       a3 c  p floppy linkage block
;       a5 c  p map
;
;       status return 0, ERR.NC or ERR.MCHK
;
;---
fq_rsecl
fdr.reg reg     d2/d3/d4/a2     ;+
        movem.l fdr.reg,-(sp)
        jsr     fq_ckrw                  ; check OK to read/write
          bne.s fdr_exnc                 ; ... no
        jsr     fq_setp                  ; set physical sector
        bra.s   fdr_do
;+++
; This routine reads a sector from a Floppy disk for direct sector IO
;
;       d0 cr   cylinder + side + sector / error code
;       d6 c  p msw drive number (1 to 8)
;       a1 c  u address to read into
;       a2      WD1772          ;+
;       a3 c  p floppy linkage block
;
;       status return 0, ERR.NC or ERR.MCHK
;
;---
fq_rsecp
;-fdrp.reg reg    d2
;-        movem.l fdrp.reg,-(sp)
        bsr.s   fq_rsec
          bne.s fdrp_err
        adda.w  iod_sctl(a4),a1
fdrp_err
;-        movem.l (sp)+,fdrp.reg
        rts
;+++
; This routine reads sectors from a Floppy disk
;
;       d0 cr   sector position track+side+sector / error code
;       d6 c  p msw drive number (1 to 8)
;       a1 c  p dma address
;       a2      WD1772          ;+
;       a3 c  p floppy linkage block
;
;       status return 0, ERR.NC or ERR.MCHK
;
;---
fq_rsec
        movem.l fdr.reg,-(sp)
fdr_do
        movea.l fdd_base(a3),a2
        moveq   #3,d4                    ; retry counter
;        tas     fdd_dma(a3)             ;+ WD1772 in use
;          bne.s fdr_nc
        assert  fdd.hold,-1
        st      fdd_actm(a3)             ; hold polling task
        move.l  d0,fdd_sadd(a3)          ; sector required
        moveq   #0,d0                    ; no verify on seek
fdr_sss
        jsr     fq_sss(pc)               ; select and seek
          bne.s fdr_mchk
        moveq   #fdc.rsec,d0             ; read sector

;-        tst.b   fdd_mode(a4)            ;+ mode DD oder HD ?
;-          ble.s fdr_sd                  ;+ ..nur bei HD
;-        ori.b   #fdc.veri,d0            ;+ settle 15/2 ms
;-fdr_sd
        jsr     fq_cmd
        move.w  iod_sctl(a4),d2         ;+ $200 bytes/sector normal
        jsr     fq_read1s(pc)           ;+
          bne.s fdr_rtry                ;+
        jsr     fq_stat(pc)              ; wait for status at end of command
          beq.s fdr_exit
          blt.s fdr_mchk
fdr_rtry
        subq.b  #1,d4                    ; retry counter
          ble.s fdr_mchk
        moveq   #0,d0
        jsr     fq_rest(pc)              ; restore
        moveq   #-1,d0                   ; settle before read
        bra.s   fdr_sss

fdr_exit
;        sf      fdd_dma(a3)             ;+ WD1772 no longer in use
        move.b   #fdd.at17,fdd_actm(a3)  ; ..let it go  $7F
fdr_exnc
        movem.l (sp)+,fdr.reg
        tst.l    d0
        rts

fdr_mchk
        moveq   #err.mchk,d0
        bra.s   fdr_exit
;fdr_nc
;        moveq   #err.nc,d0
;        bra.s   fdr_exnc
        end
